<?php

namespace Five9;

class Settings {
	private static $instance;

	public static function instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Settings ) ) {
			self::$instance = new Settings;

			self::$instance->actions();
		}

		return self::$instance;
	}

	private function actions() {
		add_action( 'admin_init', array( $this, 'register_settings' ) );
		add_action( 'init', array( $this, 'register_post_meta' ) );
		add_action( 'rest_api_init', array( $this, 'register_post_meta_endpoint' ) );
	}

	public function register_settings() {
		register_setting( 'five9_settings', 'five9_settings', array( $this, 'sanitize_settings' ) );
	}

	public function register_post_meta() {
		register_meta(
			'post', '_five9_enable_chat', array(
				'type'         => 'string',
				'single'       => true,
				'show_in_rest' => true,
			)
		);
	}

	public function register_post_meta_endpoint() {
		register_rest_route(
			'five9-chat/v1', '/update-meta', array(
				'methods'  => 'POST',
				'callback' => array( $this, 'update_post_meta_callback' ),
				'args'     => array(
					'id' => array(
						'sanitize_callback' => 'absint',
					),
				),
			)
		);
	}

	public function update_post_meta_callback( $data ) {
		return update_post_meta( $data['id'], $data['key'], $data['value'] );
	}

	public function sanitize_settings( $input ) {
		$setting_types = self::chat_settings_definitions();
		foreach ( $input as $key => $value ) {
			if ( ! empty( $setting_types[ $key ]['sanitize_callback'] ) && is_callable( $setting_types[ $key ]['sanitize_callback'] ) ) {
				$input[ $key ] = $setting_types[ $key ]['sanitize_callback']( $value );
			}
		}

		return $input;
	}

	public static function get_settings() {
		$settings = get_option( 'five9_settings', array() );

		$registered_settings = self::chat_settings_definitions();
		foreach( $registered_settings as $setting_id => $setting_data ) {
			if ( ! isset( $settings[ $setting_id ] ) ) {
				$settings[ $setting_id ] = isset( $setting_data['default'] ) ? $setting_data['default'] : false;
			}
		}

		return apply_filters( 'Five9\Settings\get_settings', $settings );
	}

	public static function get_setting( $setting_name = '' ) {
		$settings = self::get_settings();
		$return   = false;

		if ( isset( $settings[ $setting_name ] ) ) {
			$return = $settings[ $setting_name ];
		}

		return apply_filters( 'Five9\Settings\get_setting', $return, $setting_name );
	}

	public static function chat_settings_definitions() {
		$settings_data = array(
			'enabled' => array(
				'label'             => __( 'Auto Enable', 'five9' ),
				'type'              => 'checkbox',
				'default'           => false,
				'sanitize_callback' => 'absint',
				'tooltip'           => __( 'Enable chat by default on every page of your site.', 'five9' ),
				'description'       => __( 'You can include the chat as a link on any page using the following shortcode<br /><code>[five9-chat text="Contact Us Now"]</code>', 'five9' ),
			),
			'widget_type' => array(
				'label'   => __( 'Widget Type', 'five9' ),
				'type'    => 'select',
				'default' => 'chat',
				'options' => array(
					'chat'      => __( 'Chat', 'five9' ),
					'email'     => __( 'Email', 'five9' ),
					'proactive' => __( 'Proactive', 'five9' ),
				),
				'sanitize_callback' => 'sanitize_text_field',
				'api_param'     => array(
					'name' => 'type',
				),
			),
			'title' => array(
				'label'       => __( 'Title', 'five9' ),
				'type'        => 'text',
				'placeholder' => __( 'Enter Title', 'five9' ),
				'tooltip'     => __( 'Title to display in the console, such as the name of your company.', 'five9' ),
				'sanitize_callback' => 'sanitize_text_field',
				'api_param'     => array(
					'name' => 'title',
				),
			),
			'domain' => array(
				'label'       => __( 'Domain', 'five9' ),
				'type'        => 'text',
				'placeholder' => __( 'Enter Domain Name', 'five9' ),
				'tooltip'     => __( 'Domain name, which you can obtain from Customer Support and the Text Channel Administration Console.', 'five9' ),
				'sanitize_callback' => 'sanitize_text_field',
				'api_param'     => array(
					'name' => 'tenant',
				),
			),
			'profiles' => array(
				'label'       => __( 'Profiles', 'five9' ),
				'type'        => 'text',
				'placeholder' => __( 'Enter Profiles', 'five9' ),
				'tooltip'     => __( 'Profiles route chat sessions and email messages to agents who are enabled for the appropriate queues. Enter either a single profile or a comma-separated list (Multiple profiles not applicable for proactive chats).', 'five9' ),
				'sanitize_callback' => 'sanitize_text_field',
				'api_param'     => array(
					'name' => 'profiles',
				),
			),
			'show_profile' => array(
				'label'   => __( 'Show Profile', 'five9' ),
				'type'    => 'checkbox',
				'default' => true,
				'sanitize_callback' => 'absint',
				'api_param'     => array(
					'name' => 'showProfile',
				),
			),
			'show_comments_field' => array(
				'label'   => __( 'Show Comments Field', 'five9' ),
				'type'    => 'checkbox',
				'default' => true,
				'sanitize_callback' => 'absint',
				'api_param'     => array(
					'group' => 'surveyOptions',
					'name' => 'showComment',
				),
			),
			'require_comments_field' => array(
				'label'   => __( 'Require Comments Field', 'five9' ),
				'type'    => 'checkbox',
				'default' => false,
				'sanitize_callback' => 'absint',
				'api_param'     => array(
					'group' => 'surveyOptions',
					'name' => 'requireComment',
				),
			),
			'show_email_button' => array(
				'label'   => __( 'Show Email Button', 'five9' ),
				'type'    => 'checkbox',
				'default' => true,
				'sanitize_callback' => 'absint',
				'api_param' => array(
					'name' => 'showEmailButton',
				),
			),
			'enable_business_hours' => array(
				'label' => __( 'Enable Business Hours', 'five9' ),
				'type' => 'checkbox',
				'default' => false,
				'sanitize_callback' => 'absint',
				'api_param' => array(
					'name' => 'useBusinessHours'
				)
			),
			'hide_widget_after_hours' => array(
				'label' => __( 'Hide Chat Widget After Hours', 'five9' ),
				'type' => 'checkbox',
				'default' => false,
				'sanitize_callback' => 'absint',
				'api_param' => array(
					'name' => 'hideDuringAfterHours'
				)
			),
			'enable_business_hours_proactive' => array(
				'label' => __( 'Hide proactive chat offer if after business hours', 'five9' ),
				'type' => 'checkbox',
				'default' => false,
				'sanitize_callback' => 'absint',
				'api_param' => array(
					'name' => 'enableBusinessHours'
				)
			),
			'show_print_button' => array(
				'label'   => __( 'Show Print Button', 'five9' ),
				'type'    => 'checkbox',
				'default' => false,
				'sanitize_callback' => 'absint',
				'api_param' => array(
					'name' => 'showPrintButton',
				),
			),
			'show_usability_menu' => array(
				'label'   => __( 'Show Accessibility Menu', 'five9' ),
				'type'    => 'checkbox',
				'default' => false,
				'sanitize_callback' => 'absint',
				'api_param' => array(
					'name' => 'allowUsabilityMenu',
				),
			),
			'profile_label' => array(
				'label'       => __( 'Profile Label', 'five9' ),
				'type'        => 'text',
				'placeholder' => __( 'Profile (Default)', 'five9' ),
				'tooltip'     => __( 'Only if profiles are shown. Replace the default label of Profile in the form with a custom word or phrase.', 'five9' ),
				'sanitize_callback' => 'sanitize_text_field',
				'api_param' => array(
					'name' => 'profileLabel',
				),
			),
			'theme' => array(
				'label'   => __( 'Theme', 'five9' ),
				'type'    => 'select',
				'default' => 'default-theme.css',
				'options' => array(
					'default-theme.css' => __( 'Default', 'five9' ),
					'flagship.css'      => __( 'Flagship', 'five9' ),
					'lime-time.css'     => __( 'Lime Time', 'five9' ),
					'high-contrast.css' => __( 'High Contrast', 'five9' ),
					'custom'            => __( 'Custom', 'five9' ),
				),
				'enables' => array(
					'setting' => 'custom_theme_css',
					'value'   => 'custom',
				),
				'sanitize_callback' => 'sanitize_text_field',
				'api_param' => array(
					'name' => 'theme',
				),
			),
			'custom_theme_css' => array(
				'label'       => __( 'Custom CSS File URL', 'five9' ),
				'type'        => 'text',
				'placeholder' => __( 'https:// path to CSS file', 'five9' ),
				'requires'    => array( 'theme' => 'custom' ),
				'sanitize_callback' => 'sanitize_text_field',
			),
			'enable_logo' => array(
				'label'   => __( 'Enable Custom Logo', 'five9' ),
				'type'    => 'checkbox',
				'default' => false,
				'enables' => array(
					'setting' => 'logo_file_url',
					'value'   => true,
				),
				'sanitize_callback' => 'absint',
			),
			'logo_file_url' => array(
				'label'       => __( 'Logo File URL', 'five9' ),
				'type'        => 'text',
				'placeholder' => __( 'https://mysite.com/logo.png', 'five9' ),
				'requires'    => array( 'enable_logo' => true ),
				'sanitize_callback' => 'sanitize_text_field',
				'api_param' => array(
					'logo',
				)
			),
			'play_sounds' => array(
				'label'   => __( 'Play Sound On New Message', 'five9' ),
				'type'    => 'checkbox',
				'default' => true,
				'sanitize_callback' => 'absint',
				'api_param' => array(
					'name' => 'playSoundOnMessage',
				),
			),
			'allow_disable_sounds' => array(
				'label'   => __( 'Allow Customer To Disable Sound', 'five9' ),
				'type'    => 'checkbox',
				'default' => false,
				'sanitize_callback' => 'absint',
				'api_param' => array(
					'name' => 'allowCustomerToControlSoundPlay',
				)
			),
			'enable_callback' => array(
				'label'   => __( 'Enable Callback', 'five9' ),
				'type'    => 'checkbox',
				'default' => false,
				'sanitize_callback' => 'absint',
				'api_param' => array(
					'name' => 'enableCallback',
				)
			),
			'callback_list' => array(
				'label'       => __( 'Callback Dialing List', 'five9' ),
				'type'        => 'text',
				'placeholder' => 'outbound_list',
				'sanitize_callback' => 'sanitize_text_field',
				'api_param' => array(
					'name' => 'callbackList',
				),
			),
			'enable_custom_language' => array(
				'label'   => __( 'Enable', 'five9' ),
				'type'    => 'checkbox',
				'default' => false,
				'enables' => array(
					'setting' => 'custom_language_url',
					'value'   => true,
				),
				'sanitize_callback' => 'absint',
			),
			'custom_language_url' => array(
				'label'       => __( 'Language File URL', 'five9' ),
				'type'        => 'text',
				'placeholder' => __( 'https:// path to JavaScript file', 'five9' ),
				'requires'    => array( 'enable_custom_language' => true ),
				'sanitize_callback' => 'sanitize_text_field',
				'api_param' => array(
					'name' => 'lang',
				),
			),
			'namespace' => array(
				'label'       => __( 'Namespace', 'five9' ),
				'type'        => 'text',
				'placeholder' => '',
				'sanitize_callback' => 'sanitize_text_field',
				'api_param' => array(
					'name' => 'namespace',
				),
			),
			'auto_start' => array(
				'label'   => __( 'Auto Start if Name & Email supplied', 'five9' ),
				'type'    => 'select',
				'default' => 'true',
				'options' => array(
					'true'  => __( 'True (default)', 'five9' ),
					'false' => __( 'False', 'five9' ),
				),
				'sanitize_callback' => 'sanitize_text_field',
				'api_param' => array(
					'name' => 'autostart',
				),
			),
			'rest_api_url' => array(
				'label'       => __( 'REST API', 'five9' ),
				'type'        => 'text',
				'placeholder' => '',
				'default'     => 'https://app.five9.com',
				'sanitize_callback' => 'sanitize_text_field',
			),
			'chat_url' => array(
				'label'       => __( 'Chat URL', 'five9' ),
				'type'        => 'text',
				'placeholder' => '',
				'default'     => 'https://app.five9.com/consoles/ChatConsole/index.html',
				'sanitize_callback' => 'sanitize_text_field',
			),
			'notification_type' => array(
				'label'   => __( 'Notification Type', 'five9' ),
				'type'    => 'select',
				'default' => 'notification',
				'options' => array(
					'notification' => __( 'Notification', 'five9' ),
					'modal'        => __( 'Modal', 'five9' ),
					'callback'     => __( 'Callback', 'five9' ),
				),
				'sanitize_callback' => 'sanitize_text_field',
			),
			'page_id' => array(
				'label'       => __( 'Page ID', 'five9' ),
				'type'        => 'text',
				'placeholder' => 'Page-1',
				'sanitize_callback' => 'sanitize_text_field',
			),
			'root_url' => array(
				'type'    => 'hidden',
				'default' => 'https://app.five9.com',
				'sanitize_callback' => 'sanitize_text_field',
			),
			'ga_id' => array(
				'label'       => __( 'Google Analytics ID', 'five9' ),
				'type'        => 'text',
				'placeholder' => 'UA-12345-12',
				'sanitize_callback' => 'sanitize_text_field',
				'api_param' => array(
					'name' => 'ga',
				),
			),
		);

		return apply_filters( 'Five9\Settings\chat_settings_definitions', $settings_data );
	}

	public function get_setting_definition( $setting_id = '' ) {
		$definitions = $this->chat_settings_definitions();
		$definition  = array();

		if ( ! empty( $definitions[ $setting_id ] ) ) {
			$definition = $definitions[ $setting_id ];
		}

		return apply_filters( 'Five9\Settings\get_setting_definition', $definition );
	}

	public static function get_settings_columns() {
		return array(
			'left' => array(
				'groups' => array(
					'required', 'after_chat_survey', 'chat_after_hours', 'print', 'usability',
				)
			),
			'right' => array(
				'groups' => array(
					'custom_profile_label', 'style_options', 'sounds', 'custom_language', 'technical_options', 'proactive_chats', 'auto_start',
				)
			)
		);
	}

	public static function get_settings_groups() {
		$settings_groups = array(
			'required' => array(
				'label'    => __( 'Required Fields', 'five9' ),
				'settings' => array(
					'enabled', 'widget_type', 'title', 'domain', 'profiles', 'show_profile',
				),
				'support' => array(
					'chat', 'email', 'proactive',
				)
			),
			'after_chat_survey' => array(
				'label'    => __( 'After-Chat Survey', 'five9' ),
				'tooltip'  => __( 'Enables comment options for the survey if is enabled in the Text Channel Administration Console.', 'five9' ),
				'settings' => array(
					'show_comments_field', 'require_comments_field',
				),
				'support' => array(
					'chat',
				)
			),
			'chat_after_hours' => array(
				'label'    => __( 'Chat After Hours', 'five9' ),
				'tooltip'  => __( 'Options regarding how the chat should behave if no agents are available or it is after business hours.', 'five9' ),
				'settings' => array(
					'show_email_button',
					'enable_business_hours',
					'hide_widget_after_hours'
				),
				'support' => array(
					'chat',
				)
			),
			'print' => array(
				'label'    => __( 'Print', 'five9' ),
				'tooltip'  => __( 'Enables an option in the client to print the chat transcript.', 'five9' ),
				'settings' => array(
					'show_print_button'
				),
				'support' => array(
					'chat',
				)
			),
			'usability' => array(
				'label'    => __( 'Accessibility', 'five9' ),
				'tooltip'  => __( 'Enables options in the client to adjust the type size and increase the contrast.', 'five9' ),
				'settings' => array(
					'show_usability_menu'
				),
				'support' => array(
					'chat',
				)
			),
			'custom_profile_label' => array(
				'label'    => __( 'Custom Profile Label', 'five9' ),
				'settings' => array(
					'profile_label'
				),
				'support' => array(
					'chat',
				)
			),
			'style_options' => array(
				'label'    => __( 'Style Options', 'five9' ),
				'settings' => array(
					'theme', 'custom_theme_css','enable_logo', 'logo_file_url',
				),
				'support' => array(
					'chat', 'email',
				)
			),
			'sounds' => array(
				'label'    => __( 'Sounds', 'five9' ),
				'tooltip'  => __( 'Enables customers to control sounds in the chat client.', 'five9' ),
				'settings' => array(
					'play_sounds', 'allow_disable_sounds',
				),
				'support' => array(
					'chat',
				)
			),
			'custom_language' => array(
				'label'    => __( 'Custom Language File', 'five9' ),
				'settings' => array(
					'enable_custom_language', 'custom_language_url',
				),
				'support' => array(
					'chat',
				)
			),
			'technical_options' => array(
				'label'    => __( 'Technical Options', 'five9' ),
				'tooltip'  => __( 'Misc Options', 'five9' ),
				'settings' => array(
					'namespace', 'ga_id',
				),
				'support' => array(
					'chat',
				)
			),
			'callback_options' => array(
				'label'    => __( 'Callback Options', 'five9' ),
				'tooltip'  => __( 'Allow to schedule a callback if chat is not available', 'five9' ),
				'settings' => array(
					'enable_callback', 'callback_list'
				),
				'support' => array(
					'chat',
				)
			),
			'proactive_chats' => array(
				'label' => __( 'Proactive Chat', 'five9' ),
				'settings' => array(
					'rest_api_url', 'chat_url', 'notification_type', 'page_id', 'enable_business_hours_proactive'
				),
				'support' => array(
					'proactive',
				)
			),
			'auto_start' => array(
				'label' => __( 'Auto Start Settings', 'five9' ),
				'settings' => array(
					'auto_start',
				),
				'support' => array(
					'proactive', 'chat',
				)
			)
		);

		return apply_filters( 'Five9\Settings\get_settings_groups', $settings_groups );
	}

}

Settings::instance();
